/**
 * This code was generated by
 * \ / _    _  _|   _  _
 *  | (_)\/(_)(_|\/| |(/_  v1.0.0
 *       /       /
 */

import Page = require('../../../base/Page');
import Response = require('../../../http/response');
import V1 = require('../V1');
import { SerializableClass } from '../../../interfaces';

/**
 * Initialize the UseCaseList
 *
 * PLEASE NOTE that this class contains beta products that are subject to change.
 * Use them with caution.
 *
 * @param version - Version of the resource
 */
declare function UseCaseList(version: V1): UseCaseListInstance;

interface UseCaseListInstance {
  /**
   * Streams UseCaseInstance records from the API.
   *
   * This operation lazily loads records as efficiently as possible until the limit
   * is reached.
   *
   * The results are passed into the callback function, so this operation is memory
   * efficient.
   *
   * If a function is passed as the first argument, it will be used as the callback
   * function.
   *
   * @param callback - Function to process each record
   */
  each(callback?: (item: UseCaseInstance, done: (err?: Error) => void) => void): void;
  /**
   * Streams UseCaseInstance records from the API.
   *
   * This operation lazily loads records as efficiently as possible until the limit
   * is reached.
   *
   * The results are passed into the callback function, so this operation is memory
   * efficient.
   *
   * If a function is passed as the first argument, it will be used as the callback
   * function.
   *
   * @param opts - Options for request
   * @param callback - Function to process each record
   */
  each(opts?: UseCaseListInstanceEachOptions, callback?: (item: UseCaseInstance, done: (err?: Error) => void) => void): void;
  /**
   * Retrieve a single target page of UseCaseInstance records from the API.
   *
   * The request is executed immediately.
   *
   * If a function is passed as the first argument, it will be used as the callback
   * function.
   *
   * @param callback - Callback to handle list of records
   */
  getPage(callback?: (error: Error | null, items: UseCasePage) => any): Promise<UseCasePage>;
  /**
   * Retrieve a single target page of UseCaseInstance records from the API.
   *
   * The request is executed immediately.
   *
   * If a function is passed as the first argument, it will be used as the callback
   * function.
   *
   * @param targetUrl - API-generated URL for the requested results page
   * @param callback - Callback to handle list of records
   */
  getPage(targetUrl?: string, callback?: (error: Error | null, items: UseCasePage) => any): Promise<UseCasePage>;
  /**
   * Lists UseCaseInstance records from the API as a list.
   *
   * If a function is passed as the first argument, it will be used as the callback
   * function.
   *
   * @param callback - Callback to handle list of records
   */
  list(callback?: (error: Error | null, items: UseCaseInstance[]) => any): Promise<UseCaseInstance[]>;
  /**
   * Lists UseCaseInstance records from the API as a list.
   *
   * If a function is passed as the first argument, it will be used as the callback
   * function.
   *
   * @param opts - Options for request
   * @param callback - Callback to handle list of records
   */
  list(opts?: UseCaseListInstanceOptions, callback?: (error: Error | null, items: UseCaseInstance[]) => any): Promise<UseCaseInstance[]>;
  /**
   * Retrieve a single page of UseCaseInstance records from the API.
   *
   * The request is executed immediately.
   *
   * If a function is passed as the first argument, it will be used as the callback
   * function.
   *
   * @param callback - Callback to handle list of records
   */
  page(callback?: (error: Error | null, items: UseCasePage) => any): Promise<UseCasePage>;
  /**
   * Retrieve a single page of UseCaseInstance records from the API.
   *
   * The request is executed immediately.
   *
   * If a function is passed as the first argument, it will be used as the callback
   * function.
   *
   * @param opts - Options for request
   * @param callback - Callback to handle list of records
   */
  page(opts?: UseCaseListInstancePageOptions, callback?: (error: Error | null, items: UseCasePage) => any): Promise<UseCasePage>;
  /**
   * Provide a user-friendly representation
   */
  toJSON(): any;
}

/**
 * Options to pass to each
 *
 * @property callback -
 *                         Function to process each record. If this and a positional
 *                         callback are passed, this one will be used
 * @property done - Function to be called upon completion of streaming
 * @property limit -
 *                         Upper limit for the number of records to return.
 *                         each() guarantees never to return more than limit.
 *                         Default is no limit
 * @property pageSize -
 *                         Number of records to fetch per request,
 *                         when not set will use the default value of 50 records.
 *                         If no pageSize is defined but a limit is defined,
 *                         each() will attempt to read the limit with the most efficient
 *                         page size, i.e. min(limit, 1000)
 */
interface UseCaseListInstanceEachOptions {
  callback?: (item: UseCaseInstance, done: (err?: Error) => void) => void;
  done?: Function;
  limit?: number;
  pageSize?: number;
}

/**
 * Options to pass to list
 *
 * @property limit -
 *                         Upper limit for the number of records to return.
 *                         list() guarantees never to return more than limit.
 *                         Default is no limit
 * @property pageSize -
 *                         Number of records to fetch per request,
 *                         when not set will use the default value of 50 records.
 *                         If no page_size is defined but a limit is defined,
 *                         list() will attempt to read the limit with the most
 *                         efficient page size, i.e. min(limit, 1000)
 */
interface UseCaseListInstanceOptions {
  limit?: number;
  pageSize?: number;
}

/**
 * Options to pass to page
 *
 * @property pageNumber - Page Number, this value is simply for client state
 * @property pageSize - Number of records to return, defaults to 50
 * @property pageToken - PageToken provided by the API
 */
interface UseCaseListInstancePageOptions {
  pageNumber?: number;
  pageSize?: number;
  pageToken?: string;
}

interface UseCasePayload extends UseCaseResource, Page.TwilioResponsePayload {
}

interface UseCaseResource {
  code: string;
  description: string;
  name: string;
}

interface UseCaseSolution {
}


declare class UseCaseInstance extends SerializableClass {
  /**
   * Initialize the UseCaseContext
   *
   * PLEASE NOTE that this class contains beta products that are subject to change.
   * Use them with caution.
   *
   * @param version - Version of the resource
   * @param payload - The instance payload
   */
  constructor(version: V1, payload: UseCasePayload);

  code: string;
  description: string;
  name: string;
  /**
   * Provide a user-friendly representation
   */
  toJSON(): any;
}


declare class UseCasePage extends Page<V1, UseCasePayload, UseCaseResource, UseCaseInstance> {
  /**
   * Initialize the UseCasePage
   *
   * PLEASE NOTE that this class contains beta products that are subject to change.
   * Use them with caution.
   *
   * @param version - Version of the resource
   * @param response - Response from the API
   * @param solution - Path solution
   */
  constructor(version: V1, response: Response<string>, solution: UseCaseSolution);

  /**
   * Build an instance of UseCaseInstance
   *
   * @param payload - Payload response from the API
   */
  getInstance(payload: UseCasePayload): UseCaseInstance;
  /**
   * Provide a user-friendly representation
   */
  toJSON(): any;
}

export { UseCaseInstance, UseCaseList, UseCaseListInstance, UseCaseListInstanceEachOptions, UseCaseListInstanceOptions, UseCaseListInstancePageOptions, UseCasePage, UseCasePayload, UseCaseResource, UseCaseSolution }
